# -*- coding: utf-8 -*-

# Copyright 2015 OpenMarket Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from unpaddedbase64 import decode_base64

import nacl.signing

from signedjson.sign import sign_json

SIGNING_KEY_SEED = decode_base64(
    "YJDBA9Xnr2sVqXD9Vj7XVUnmFZcZrlw8Md7kMW+3XA1"
)

KEY_ALG = "ed25519"
KEY_VER = 1
KEY_NAME = "%s:%d" % (KEY_ALG, KEY_VER)


class KnownKeyTestCase(unittest.TestCase):
    """ An entirely deterministic test using a given signing key seed, so that
    other implementations can compare that they get the same result. """

    def setUp(self):
        self.signing_key = nacl.signing.SigningKey(SIGNING_KEY_SEED)
        self.signing_key.alg = KEY_ALG
        self.signing_key.version = KEY_VER

    def test_sign_minimal(self):
        self.assertEquals(
            sign_json({}, "domain", self.signing_key),
            {
                'signatures': {
                    'domain': {
                        KEY_NAME: "K8280/U9SSy9IVtjBuVeLr+HpOB4BQFWbg+UZaADMt"
                        "TdGYI7Geitb76LTrr5QV/7Xg4ahLwYGYZzuHGZKM5ZAQ"
                    },
                }
            }
        )

    def test_sign_with_data(self):
        self.assertEquals(
            sign_json({'one': 1, 'two': "Two"}, "domain", self.signing_key),
            {
                'one': 1,
                'two': "Two",
                'signatures': {
                    'domain': {
                        KEY_NAME: "KqmLSbO39/Bzb0QIYE82zqLwsA+PDzYIpIRA2sRQ4s"
                        "L53+sN6/fpNSoqE7BP7vBZhG6kYdD13EIMJpvhJI+6Bw"
                    },
                }
            }
        )



# -*- coding: utf-8 -*-

# Copyright 2014 OpenMarket Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from unpaddedbase64 import encode_base64

from signedjson.sign import (
    sign_json, verify_signed_json, signature_ids, SignatureVerifyException
)


class JsonSignTestCase(unittest.TestCase):
    def setUp(self):
        self.message = {'foo': 'bar', 'unsigned': {}}
        self.sigkey = MockSigningKey()
        self.assertEqual(self.sigkey.alg, 'mock')
        self.signed = sign_json(self.message, 'Alice', self.sigkey)
        self.verkey = MockVerifyKey()

    def test_sign_and_verify(self):
        self.assertIn('signatures', self.signed)
        self.assertIn('Alice', self.signed['signatures'])
        self.assertIn('mock:test', self.signed['signatures']['Alice'])
        self.assertEqual(
            self.signed['signatures']['Alice']['mock:test'],
            encode_base64(b'x_______')
        )
        self.assertEqual(self.sigkey.signed_bytes, b'{"foo":"bar"}')
        verify_signed_json(self.signed, 'Alice', self.verkey)

    def test_signature_ids(self):
        key_ids = signature_ids(
            self.signed, 'Alice', supported_algorithms=['mock']
        )
        self.assertListEqual(key_ids, ['mock:test'])

    def test_verify_fail(self):
        self.signed['signatures']['Alice']['mock:test'] = encode_base64(
            b'not a signature'
        )
        with self.assertRaises(SignatureVerifyException):
            verify_signed_json(self.signed, 'Alice', self.verkey)

    def test_verify_fail_no_signatures(self):
        with self.assertRaises(SignatureVerifyException):
            verify_signed_json({}, 'Alice', self.verkey)

    def test_verify_fail_no_signature_for_alice(self):
        with self.assertRaises(SignatureVerifyException):
            verify_signed_json({'signatures': {}}, 'Alice', self.verkey)

    def test_verify_fail_not_base64(self):
        invalid = {'signatures': {'Alice': {'mock:test': 'not base64'}}}
        with self.assertRaises(SignatureVerifyException):
            verify_signed_json(invalid, 'Alice', self.verkey)


class MockSigningKey(object):
    alg = "mock"
    version = "test"

    def sign(self, signed_bytes):
        self.signed_bytes = signed_bytes
        return MockSignature()


class MockVerifyKey(object):
    alg = "mock"
    version = "test"

    def verify(self, message, sig):
        if not sig == b"x_______":
            raise Exception()


class MockSignature(object):
    def __init__(self):
        self.signature = b"x_______"



import unittest

from signedjson.key import (
    decode_signing_key_base64,
    decode_verify_key_base64,
    decode_verify_key_bytes,
    encode_signing_key_base64,
    encode_verify_key_base64,
    generate_signing_key,
    get_verify_key,
    is_signing_algorithm_supported,
    read_old_signing_keys,
    read_signing_keys,
    write_signing_keys,
)


class GenerateTestCase(unittest.TestCase):
    def test_generate_key(self):
        my_version = "my_version"
        my_key = generate_signing_key(my_version)
        self.assertEquals(my_key.alg, "ed25519")
        self.assertEquals(my_key.version, my_version)


class DecodeTestCase(unittest.TestCase):
    def setUp(self):
        self.version = "my_version"
        self.key = generate_signing_key(self.version)
        self.key_base64 = encode_signing_key_base64(self.key)
        self.verify_key = get_verify_key(self.key)
        self.verify_key_base64 = encode_verify_key_base64(self.verify_key)

    def test_decode(self):
        decoded_key = decode_signing_key_base64(
            "ed25519", self.version, self.key_base64
        )
        self.assertEquals(decoded_key.alg, "ed25519")
        self.assertEquals(decoded_key.version, self.version)

    def test_decode_invalid_base64(self):
        with self.assertRaises(Exception):
            decode_signing_key_base64("ed25519", self.version, "not base 64")

    def test_decode_signing_invalid_algorithm(self):
        with self.assertRaises(Exception):
            decode_signing_key_base64("not a valid alg", self.version, "")

    def test_decode_invalid_key(self):
        with self.assertRaises(Exception):
            decode_signing_key_base64("ed25519", self.version, "")

    def test_decode_verify_key(self):
        decoded_key = decode_verify_key_base64(
            "ed25519", self.version, self.verify_key_base64
        )
        self.assertEquals(decoded_key.alg, "ed25519")
        self.assertEquals(decoded_key.version, self.version)

    def test_decode_verify_key_invalid_base64(self):
        with self.assertRaises(Exception):
            decode_verify_key_base64("ed25519", self.version, "not base 64")

    def test_decode_verify_key_invalid_algorithm(self):
        with self.assertRaises(Exception):
            decode_verify_key_base64("not a valid alg", self.version, "")

    def test_decode_verify_key_invalid_key(self):
        with self.assertRaises(Exception):
            decode_verify_key_base64("ed25519", self.version, "")

    def test_read_keys(self):
        stream = ["ed25519 %s %s" % (self.version, self.key_base64)]
        keys = read_signing_keys(stream)
        self.assertEquals(len(keys), 1)

    def test_read_old_keys(self):
        stream = ["ed25519 %s 0 %s" % (self.version, self.verify_key_base64)]
        keys = read_old_signing_keys(stream)
        self.assertEquals(len(keys), 1)

    def test_decode_verify_invalid_algorithm(self):
        with self.assertRaises(Exception):
            decode_verify_key_bytes("not a valid alg", self.verify_key)

    def test_write_signing_keys(self):
        class MockStream(object):
            def write(self, data):
                pass

        write_signing_keys(MockStream(), [self.key])


class AlgorithmSupportedTestCase(unittest.TestCase):
    def test_ed25519(self):
        self.assertTrue(is_signing_algorithm_supported("ed25519:an_id"))

    def test_unsupported(self):
        self.assertFalse(is_signing_algorithm_supported("unsupported:"))



#!/usr/bin/env python

# Copyright 2015 OpenMarket Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup
from codecs import open
import os

here = os.path.abspath(os.path.dirname(__file__))


def read_file(path_segments):
    """Read a UTF-8 file from the package. Takes a list of strings to join to
    make the path"""
    file_path = os.path.join(here, *path_segments)
    with open(file_path, encoding="utf-8") as f:
        return f.read()


setup(
    name="signedjson",
    packages=["signedjson"],
    description="Sign JSON with Ed25519 signatures",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    install_requires=[
        "canonicaljson>=1.0.0",
        "unpaddedbase64>=1.0.1",
        "pynacl>=0.3.0",
        "typing_extensions>=3.5",
        'typing>=3.5;python_version<"3.5"',
        "importlib_metadata",
    ],
    long_description=read_file(("README.rst",)),
    keywords="json",
)
